using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace EventLogClient
{
    public partial class EventLogMainWindow : Form
    {
        private bool inTray = false;
        private bool minimizeMessageShown = false;
        private bool quitnow = false;
        private String lastClickedSource = "asdfsadfasdf";

        private List<EventLogService.EventEntry> EventViewerEvents;
        private List<EventLogService.EventEntry> FilterEvents;

        private static List<EventlogSource> EventlogSources = new List<EventlogSource>();

        #region Window and Systray Management
        public EventLogMainWindow()
        {
            InitializeComponent();
        }

        #region Systray
        private void showToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Show();
            inTray = false;
        }

        private void quitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            quitnow = true;
            this.Close();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void trayIcon_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            this.Show();
            inTray = false;
        }
        #endregion

        #region Closing to SystemTray
        private void EventLogMainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!quitnow)
            {
                if (!inTray)
                {
                    if (this.WindowState != FormWindowState.Minimized)
                    {
                        inTray = true;
                        //we don't close the app...
                        e.Cancel = true;
                        //minimize the app and then display a message to the user so
                        //they understand they didn't close the app they just sent it to the tray.
                        //this.WindowState = FormWindowState.Minimized;
                        this.Hide();
                        //Show the message.
                        if (!minimizeMessageShown)
                        {
                            minimizeMessageShown = true;

                            trayIcon.ShowBalloonTip(1000, "EventLog Client",
                                 "You have not closed this appliation." +
                                 (Char)(13) + "It has be moved to the tray." +
                                 (Char)(13) + "Right click the Icon to close it.",
                                 ToolTipIcon.Info);
                        }
                    }
                }
            }
        }
        #endregion

        #endregion

        #region Sources Tab

        #region Refresh the Available Box
        private void RefreshButton_Click(object sender, EventArgs e)
        {
            #region Check Values
            if (ServerURLBox.Text == "")
            {
                MessageBox.Show("Error: You should enter a source-URL first.");
                return;
            }
            String URL = ServerURLBox.Text;
            #endregion

            #region Connect and Handle...
            string[] logsources = GetAvailableLogSources(ServerURLBox.Text);

            availableEventLogs.Items.Clear();

            foreach (String logsource in logsources)
            {
                availableEventLogs.Items.Add(logsource);
            }
            #endregion
        }
        #endregion

        #region Add Source
        private void addSourceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // enable the controls
            splitContainer1.Panel2.Enabled = true;
            addToSourcesListButton.Enabled = true;
            saveButton.Enabled = false;

            // clear the stuff
            availableEventLogs.Items.Clear();
            ServerNameBox.Text = "";
            ServerURLBox.Text = "";
        }
        #endregion

        #region AddToSourcesListButton
        private void addToSourcesListButton_Click(object sender, EventArgs e)
        {
            #region Check Values
            if (ServerURLBox.Text == "")
            {
                MessageBox.Show("Error: You should enter a source-URL.");
                return;
            }
            String URL = ServerURLBox.Text;

            if (ServerNameBox.Text == "")
            {
                MessageBox.Show("Error: You should enter a servername.");
                return;
            }
            if (availableEventLogs.Items.Count == 0)
            {
                RefreshButton_Click(sender, e);
                if (availableEventLogs.Items.Count == 0)
                {
                    //MessageBox.Show("Error: Your source-URL seems not to be valid.");
                    return;
                }
            }
            foreach (EventlogSource source in EventlogSources)
            {
                if (source.ServerName == ServerNameBox.Text)
                {
                    MessageBox.Show("Error: There's already a server with that name in the list");
                    return;
                }
            }
            #endregion

            #region create the eventlog data structure
            EventlogSource newEventlogSource = new EventlogSource();

            Eventlog newlog;

            for (int i = 0; i <= availableEventLogs.Items.Count-1; i++)
            {
                newlog = new Eventlog();

                newlog.selected = availableEventLogs.GetItemChecked(i);
                newlog.EventlogName = (String)availableEventLogs.Items[i];
                
                newEventlogSource.Eventlogs.Add(newlog);
            }
            newEventlogSource.ServerName = ServerNameBox.Text;
            newEventlogSource.ServerURL = ServerURLBox.Text;
            #endregion

            #region Add the newEventlog to the list
            EventlogSources.Add(newEventlogSource);
            RenderAllEventlogSources();
            #endregion

            #region Select the newly added item
            //availableEventLogs.SelectedIndex = availableEventLogs.Items.Count;
            #endregion
        }
        #endregion

        #region Render Eventlog Sourceslist
        private void RenderAllEventlogSources()
        {
            SourcesList.Items.Clear();
            sourcesDropDown.Items.Clear();
            eventLogViewerSourceSelection.Items.Clear();

            foreach (EventlogSource source in EventlogSources)
            {
                SourcesList.Items.Add(source.ServerName);
                sourcesDropDown.Items.Add(source.ServerName);
                eventLogViewerSourceSelection.Items.Add(source.ServerName);
            }
        }
        #endregion

        #region When you click something in the sourceslist
        private void SourcesList_SelectedIndexChanged(object sender, EventArgs e)
        {
            EventlogSource selectedEventlog = new EventlogSource();

            if (SourcesList.SelectedItems.Count == 0) return;
            #region find the currently selected source in the data
            foreach (EventlogSource source in EventlogSources)
            {
                if (source.ServerName == SourcesList.SelectedItems[0].Text)
                {
                    selectedEventlog = source;
                    break;
                }
            }
            #endregion

            #region render it
            lastClickedSource = selectedEventlog.ServerName;

            ServerNameBox.Text = selectedEventlog.ServerName;
            ServerURLBox.Text = selectedEventlog.ServerURL;

            availableEventLogs.Items.Clear();

            int number;
            foreach (Eventlog log in selectedEventlog.Eventlogs)
            {
                number = availableEventLogs.Items.Add(log.EventlogName);
                availableEventLogs.SetItemChecked(number, log.selected);
            }
            
            splitContainer1.Panel2.Enabled = true;
            addToSourcesListButton.Enabled = false;
            saveButton.Enabled = true;
            
            #endregion
        }
        #endregion

        #region When you click the save button
        private void saveButton_Click(object sender, EventArgs e)
        {
            #region Check Values
            if (ServerURLBox.Text == "")
            {
                MessageBox.Show("Error: You should enter a source-URL.");
                return;
            }
            String URL = ServerURLBox.Text;

            if (ServerNameBox.Text == "")
            {
                MessageBox.Show("Error: You should enter a servername.");
                return;
            }
            if (availableEventLogs.Items.Count == 0)
            {
                RefreshButton_Click(sender, e);
                if (availableEventLogs.Items.Count == 0)
                {
                    //MessageBox.Show("Error: Your source-URL seems not to be valid.");
                    return;
                }
            }
            foreach (EventlogSource source in EventlogSources)
            {
                if (source.ServerName == ServerNameBox.Text)
                {
                    if (source.ServerName != lastClickedSource)
                    {
                        MessageBox.Show("Error: There's already a server with that name in the list");
                        return;
                    }
                }
            }
            #endregion

            EventlogSource selectedEventlog = new EventlogSource();

            if (SourcesList.SelectedItems.Count == 0)
            {
                MessageBox.Show("Error: Nothing was selected. Actually you shouldn't be able to click that button...");
                return;
            }

            #region create the eventlog data structure
            EventlogSource newEventlogSource = new EventlogSource();

            Eventlog newlog;

            for (int i = 0; i <= availableEventLogs.Items.Count - 1; i++)
            {
                newlog = new Eventlog();

                newlog.selected = availableEventLogs.GetItemChecked(i);
                newlog.EventlogName = (String)availableEventLogs.Items[i];

                newEventlogSource.Eventlogs.Add(newlog);
            }
            newEventlogSource.ServerName = ServerNameBox.Text;
            newEventlogSource.ServerURL = ServerURLBox.Text;
            #endregion

            #region change it...
            EventlogSources[SourcesList.SelectedIndices[0]] = newEventlogSource;
            #endregion

            #region Render it
            RenderAllEventlogSources();
            #endregion
        }
        #endregion

        #endregion

        #region EventLog Viewer Tab
        #region When you click on the EventLogViewer Selection
        private void eventLogViewerSourceSelection_SelectedIndexChanged(object sender, EventArgs e)
        {
            EventlogSource selectedEventlog = new EventlogSource();

            if ((String)eventLogViewerSourceSelection.SelectedItem == "")
            {
                refreshEventLogViewerButton.Enabled = false;
                return;
            }
            #region find the currently selected source in the data
            foreach (EventlogSource source in EventlogSources)
            {
                if (source.ServerName == (String)eventLogViewerSourceSelection.SelectedItem)
                {
                    selectedEventlog = source;
                    break;
                }
            }
            #endregion

            #region get Webservice, connect and get all the logs
            EventViewerEvents = GetTodaysEntries(selectedEventlog);
            #endregion

            #region Render it..
            EventViewerListview.Items.Clear();
            foreach (EventLogService.EventEntry entry in EventViewerEvents)
            {
                ListViewItem newItem = EventViewerListview.Items.Add(entry.TimeGenerated.ToShortTimeString());

                newItem.SubItems.Add(entry.Message);
            }
            refreshEventLogViewerButton.Enabled = true;
            #endregion
        }
        #endregion

        #region When you click on the refresh Logviewer Button
        private void refreshEventLogViewerButton_Click(object sender, EventArgs e)
        {
            eventLogViewerSourceSelection_SelectedIndexChanged(sender, e);
        }
        #endregion
        #endregion

        #region WebService Consumers

        #region GetTodaysEntries
        public List<EventLogService.EventEntry> GetTodaysEntries(EventlogSource EventlogSource_)
        {
            EventLogService.EventLogService Service = new EventLogClient.EventLogService.EventLogService();
            List<EventLogService.EventEntry> Output = new List<EventLogClient.EventLogService.EventEntry>();

            try
            {
                Service.Url = EventlogSource_.ServerURL;
            }
            catch (Exception)
            {
                MessageBox.Show("Error: You should enter a valid source-URL first.");
                return Output;
            }

            foreach (Eventlog log in EventlogSource_.Eventlogs)
            {
                if (log.selected)
                {
                    EventLogService.EventEntry[] Events = new EventLogService.EventEntry[0];
                    try
                    {
                        Events = Service.GetTodaysEntries(log.EventlogName);
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Error: The URL you specified is not valid or the log " + log.EventlogName + "is not available anymore");
                        return Output;
                    }

                    foreach (EventLogService.EventEntry entry in Events)
                    {
                        Output.Add(entry);
                    }
                }
            }

            return Output;

        }
        #endregion

        #region GetAvailableLogSources
        public String[] GetAvailableLogSources(String URL)
        {
            EventLogService.EventLogService Service = new EventLogClient.EventLogService.EventLogService();

            try
            {
                Service.Url = URL;
            }
            catch (Exception)
            {
                MessageBox.Show("Error: You should enter a valid source-URL first.");
                return new String[0];
            }
            string[] logsources = new string[0];
            try
            {
                logsources = Service.GetEventLogSources();
            }
            catch (Exception)
            {
                MessageBox.Show("Error: The URL you specified is not valid.");
                return new String[0];
            }

            return logsources;
        }
        #endregion

        #endregion

        #region Filters Tab
        #region Add Filter Context Menu Click
        private void addFilterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Filtername.Text = "";
            FilterSettingsGroupBox.Enabled = true;
            addToFilterListButton.Enabled = true;
            FilterEventLog.Enabled = true;
        }
        #endregion

        #region When you click on the FilterSources Dropdown...
        private void sourcesDropDown_SelectedIndexChanged(object sender, EventArgs e)
        {
            EventlogSource selectedEventlog = new EventlogSource();

            if ((String)sourcesDropDown.SelectedItem != "")
            {
                FilterSettingsGroupBox.Enabled = false;
                FilterListingBox.Enabled = true;

                #region find the currently selected source in the data
                foreach (EventlogSource source in EventlogSources)
                {
                    if (source.ServerName == (String)sourcesDropDown.SelectedItem)
                    {
                        selectedEventlog = source;
                        break;
                    }
                }
                #endregion

                #region Now render the FilterListingBox...
                FiltersListView.Items.Clear();

                foreach (Eventlog elog in selectedEventlog.Eventlogs)
                {
                    foreach (EventLogFilter filter in elog.Filters)
                    {
                        FiltersListView.Items.Add(elog.EventlogName+":"+filter.Filtername);
                    }
                }
                #endregion

                #region Now render the FilterEventLog Box...
                FilterEventLog.Items.Clear();

                foreach (Eventlog log in selectedEventlog.Eventlogs)
                {
                    if(log.selected) FilterEventLog.Items.Add(log.EventlogName);
                }

                #endregion
                return;
            }
            else
            {
                FilterSettingsGroupBox.Enabled = false;
                FilterListingBox.Enabled = false;

                return;
            }
        }
        #endregion

        #region When you click on the PropertyCombobox
        private void propertyValueComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            EventlogSource selectedEventlog = new EventlogSource();

            if ((String)sourcesDropDown.SelectedItem != "")
            {
                #region find the currently selected source in the data
                foreach (EventlogSource source in EventlogSources)
                {
                    if (source.ServerName == (String)sourcesDropDown.SelectedItem)
                    {
                        selectedEventlog = source;
                        break;
                    }
                }
                #endregion

                #region get Webservice, connect and get all the logs
                FilterEvents = GetTodaysEntries(selectedEventlog);
                #endregion   

                List<String> preSuggest = new List<string>();

                filterValueCombobox.Items.Clear();

                switch ((String)propertyValueComboBox.SelectedItem)
                {
                    case "Category":
                        foreach (EventLogService.EventEntry entry in FilterEvents)
                        {
                            preSuggest.Add(entry.Category);
                        }
                        break;
                    case "EventID":
                        foreach (EventLogService.EventEntry entry in FilterEvents)
                        {
                            preSuggest.Add(entry.EventID.ToString());
                        }
                        break;
                    case "MachineName":
                        foreach (EventLogService.EventEntry entry in FilterEvents)
                        {
                            preSuggest.Add(entry.MachineName);
                        }
                        break;
                    case "Message":
                        foreach (EventLogService.EventEntry entry in FilterEvents)
                        {
                            preSuggest.Add(entry.Message);
                        }
                        break;
                    case "Source":
                        foreach (EventLogService.EventEntry entry in FilterEvents)
                        {
                            preSuggest.Add(entry.Source);
                        }
                        break;
                    case "Username":
                        foreach (EventLogService.EventEntry entry in FilterEvents)
                        {
                            preSuggest.Add(entry.Username);
                        }
                        break;
                }
                preSuggest = StripDuplicates(preSuggest);

                foreach (String element in preSuggest)
                {
                    filterValueCombobox.Items.Add(element);
                }
            }
            else
            {
                return;
            }
        }

        public List<String> StripDuplicates(List<String> DuplicateList)
        {
            List<String> Output = new List<string>();

            foreach (String input in DuplicateList)
            {
                if (input != null)
                {
                    if (!Output.Contains(input)) Output.Add(input);
                }
            }

            return Output;
        }
        #endregion

        #region When you click on the Add To Filter List button
        private void addToFilterListButton_Click(object sender, EventArgs e)
        {
            EventLogFilter newFilter = new EventLogFilter();
            EventlogSource selectedEventlogSource = new EventlogSource();
            Eventlog selectedEventlog = new Eventlog();

            #region Check Values
            if ((String)sourcesDropDown.SelectedItem != "")
            {
                #region find the currently selected source in the data
                foreach (EventlogSource source in EventlogSources)
                {
                    if (source.ServerName == (String)sourcesDropDown.SelectedItem)
                    {
                        selectedEventlogSource = source;
                        break;
                    }
                }
                #endregion
            }
            else
                return;

            if (Filtername.Text == "")
            {
                MessageBox.Show("Error: You should enter a Filter name.");
                return;
            }
            if ((String)FilterEventLog.SelectedItem == "")
            {
                MessageBox.Show("Error: You should select an event log.");
                return;
            }
            else
            {
                #region Find the selected Event Log
                foreach (Eventlog log in selectedEventlogSource.Eventlogs)
                {
                    if (log.EventlogName == (String)FilterEventLog.SelectedItem)
                    {
                        selectedEventlog = log;
                        break;
                    }
                }
                #endregion
            }
            if ((String)propertyValueComboBox.SelectedItem == "")
            {
                MessageBox.Show("Error: You should select a property.");
                return;
            }
            if (filterValueCombobox.Text == "")
            {
                MessageBox.Show("Error: You should enter a value.");
                return;
            }
            #endregion            

            #region generate the data structure
            newFilter.ContainsValue = filterValueCombobox.Text;
            newFilter.Filtername = Filtername.Text;
            newFilter.Property = selectedEventlog.EventlogName;
            if (noErrorRadioButton.Checked)
                newFilter.WarningLevel = 0;
            if (WarningRadioButton.Checked)
                newFilter.WarningLevel = 1;
            if (ErroRadioButton.Checked)
                newFilter.WarningLevel = 2;
            #endregion

            #region Add the filter to the eventlog..
            selectedEventlog.Filters.Add(newFilter);
            #endregion

            #region Now render the FilterListView Box...
            sourcesDropDown_SelectedIndexChanged(sender, e);
            #endregion
        }
        #endregion

        private void FiltersListView_SelectedIndexChanged(object sender, EventArgs e)
        {

        }
        #endregion
    }

    #region Data Structures
    [Serializable]
    public class Eventlog
    {
        public String EventlogName;
        public Boolean selected;
        public List<EventLogFilter> Filters;

        public Eventlog()
        {
            Filters = new List<EventLogFilter>();
        }
    }

    [Serializable]
    public class EventLogFilter
    {
        public String Filtername;
        public String Property;
        public String ContainsValue;
        public int WarningLevel;
    }

    [Serializable]
    public class EventlogSource
    {
        public String ServerName;
        public String ServerURL;
        public List<Eventlog> Eventlogs;
        public List<EventLogService.EventEntry> Events;
        public Boolean isOnline;

        public EventlogSource()
        {
            Eventlogs = new List<Eventlog>();
            Events = new List<EventLogClient.EventLogService.EventEntry>();
        }
    }
    #endregion
}