using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.ServiceModel;
using System.Runtime.Serialization;

namespace EventLogServer
{
    #region EventEntry Result Data structure
    [Serializable]
    public class EventEntry
    {
        public String Category;
        public long EventID;
        public String MachineName;
        public String Message;
        public String Source;
        public String Username;
        public DateTime TimeGenerated;
        public DateTime TimeWritten;
    }
    #endregion

    #region ServiceContract
    [ServiceContract()]
    public interface IEventLogService
    {
        [OperationContract]
        List<EventEntry >GetTodaysEntries(String Logname);

        [OperationContract]
        List<String> GetEventLogSources();
    }
    #endregion

    public class EventLogService : IEventLogService
    {
        #region GetTodaysEntries
        public List<EventEntry> GetTodaysEntries(String LogName)
        {
            List<EventEntry> Output = new List<EventEntry>();

            if (EventLog.Exists(LogName))
            {
                EventLog log = new EventLog(LogName);
                for (int i = log.Entries.Count - 1; i >= 0; i--)
                {
                    EventLogEntry entry = log.Entries[i];

                    EventEntry newEntry = new EventEntry();
                    newEntry.Category = entry.Category;
                    newEntry.EventID = entry.InstanceId;
                    newEntry.MachineName = entry.MachineName;
                    newEntry.Message = entry.Message;
                    newEntry.Source = entry.Source;
                    newEntry.TimeGenerated = entry.TimeGenerated;
                    newEntry.TimeWritten = entry.TimeWritten;

                    newEntry.Username = entry.UserName;


                    // only today
                    if (entry.TimeGenerated.Date != DateTime.Now.Date)
                        break;

                    Output.Add(newEntry);
                }
                return Output;
            }
            else
                return new List<EventEntry>();
        }
        #endregion

        #region GetEventLogSources
        public List<String> GetEventLogSources()
        {
            List<String> Output = new List<string>();

            EventLog[] AvailableEventlogs = EventLog.GetEventLogs();

            foreach (EventLog log in AvailableEventlogs)
            {
                Output.Add(log.Log);
            }

            return Output;
        }
        #endregion
    }
}
